--
-- InteractiveControl
-- Specialization for an interactive control
--
-- SFM-Modding
-- @author  Manuel Leithner
-- @date  29/12/09
--

InteractiveControl = {};

function InteractiveControl.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Steerable, specializations);
end;

function InteractiveControl:load(xmlFile)

	source(Utils.getFilename("InteractiveComponentInterface.lua", self.baseDirectory));

	self.setPanelOverlay = SpecializationUtil.callSpecializationsFunction("setPanelOverlay");

	self.interactiveObjects = {};	
	
	self.indoorCamIndex = 2;
	self.outdoorCamIndex = 1;

	self.lastMouseXPos = 0;
	self.lastMouseYPos = 0;
	self.lastXTrans = 0;
	self.lastYTrans = 0;
	self.lastZTrans = 0;

	self.panelOverlay = nil;
	self.doWrap = false;
	self.foundInteractiveObject = nil;
	self.lastCamIndex = 0;
	self.wasMouseDown = false;
	self.isMouseActive = false;
end;

function InteractiveControl:delete()
	for k,v in pairs(self.interactiveObjects) do
		v:delete();
	end;
end;

function InteractiveControl:mouseEvent(posX, posY, isDown, isUp, button)	
	self.lastMouseXPos = posX;
	self.lastMouseYPos = posY;
	if isDown then
		if Input.isMouseButtonPressed(Input.MOUSE_BUTTON_LEFT) and self.foundInteractiveObject ~= nil then			
			self.foundInteractiveObject:doAction();
		end;
		if self.camIndex == self.outdoorCamIndex then
			local outdoorCam = self.cameras[self.outdoorCamIndex];
			if Input.isMouseButtonPressed(Input.MOUSE_BUTTON_WHEEL_UP) then
				outdoorCam.zoom(outdoorCam, -0.75);
			else
				if Input.isMouseButtonPressed(Input.MOUSE_BUTTON_WHEEL_DOWN) then
					outdoorCam.zoom(outdoorCam, 0.75);
				end;
			end;
		end;	
		
		self.wasMouseDown = true;
	end;
	
	for k,v in pairs(self.interactiveObjects) do
		v:mouseEvent(posX, posY, isDown, isUp, button);
	end;
end;

function InteractiveControl:keyEvent(unicode, sym, modifier, isDown)
	for k,v in pairs(self.interactiveObjects) do
		v:keyEvent(unicode, sym, modifier, isDown);
	end;
end;

function InteractiveControl:update(dt)

	if self:getIsActive() then
		self.cameraRotationActive = false;	
		self.foundInteractiveObject = nil;
		
		if InputBinding.hasEvent(InputBinding.INTERACTIVE_CONTROL_SWITCH) then
			self.isMouseActive = not self.isMouseActive;
			
			if not self.isMouseActive then
				setShowMouseCursor(false);
				self.cameras[self.camIndex].isActivated = true;
			end;
			
			for k,v in pairs(self.interactiveObjects) do
				v:setVisible(self.isMouseActive);
			end;
		end;
		
		if self.isMouseActive then
			if self.camIndex == self.indoorCamIndex or self.camIndex == self.outdoorCamIndex then
				local indoorCam = self.cameras[self.indoorCamIndex];
				local outdoorCam = self.cameras[self.outdoorCamIndex];
				
				if Input.isMouseButtonPressed(Input.MOUSE_BUTTON_RIGHT) then
					
					if self.doWrap then
						wrapMousePosition(0.5, 0.5);
						self.lastMouseYPos = 0.5;
						self.lastMouseXPos = 0.5;
						self.doWrap = false;
					end;
					if self.lastCamIndex == self.camIndex then
						if indoorCam.posYLast ~= nil and indoorCam.posXLast ~= nil then
							indoorCam.rotX = indoorCam.rotX - (indoorCam.posYLast-self.lastMouseYPos);
							indoorCam.rotY = indoorCam.rotY - (self.lastMouseXPos-indoorCam.posXLast);
						end;
						if outdoorCam.posYLast ~= nil and outdoorCam.posXLast ~= nil and not self.wasMouseDown then
							outdoorCam.rotX = outdoorCam.rotX - (outdoorCam.posYLast-self.lastMouseYPos);
							outdoorCam.rotY = outdoorCam.rotY - (self.lastMouseXPos-outdoorCam.posXLast);
						end;
						if self.camIndex == self.outdoorCamIndex then
							setTranslation(outdoorCam.cameraNode, self.lastXTrans, self.lastYTrans, self.lastZTrans);
							outdoorCam.transX = self.lastXTrans;
							outdoorCam.transY = self.lastYTrans;
							outdoorCam.transZ = self.lastZTrans;
						end;
						
						self.wasMouseDown = false;
					end;
					self.lastCamIndex = self.camIndex;
					indoorCam.isActivated = self.camIndex == self.indoorCamIndex;
					outdoorCam.isActivated = self.camIndex == self.outdoorCamIndex;
					setShowMouseCursor(false);	
				else
					self.doWrap = true;
					indoorCam.isActivated = self.camIndex ~= self.indoorCamIndex;
					outdoorCam.isActivated = self.camIndex ~= self.outdoorCamIndex;
					setShowMouseCursor(true);	

					if self.camIndex == self.outdoorCamIndex then						
						self.lastXTrans = outdoorCam.transX;
						self.lastYTrans = outdoorCam.transY;
						self.lastZTrans = outdoorCam.transZ;
					end;					
				end;	
			else
				self.cameras[self.indoorCamIndex].isActivated = true;
				self.cameras[self.outdoorCamIndex].isActivated = true;
				setShowMouseCursor(false);
				self.lastCamIndex = self.camIndex;
			end;
		else
			self.foundInteractiveObject = nil;
		end;	
		for k,v in pairs(self.interactiveObjects) do
			v:update(dt);
			if self.isMouseActive then
				v:onExit(dt);
				if self.camIndex == self.indoorCamIndex then
					local worldX,worldY,worldZ = getWorldTranslation(v.mark);
					local x,y,z = project(worldX,worldY,worldZ);
					if z <= 1 then					
						if self.lastMouseXPos > (x-v.size/2) and self.lastMouseXPos < (x+v.size/2) then
							if self.lastMouseYPos > (y-v.size/2) and self.lastMouseYPos < (y+v.size/2) then
								local isOverlapped = false;
								
								if self.panelOverlay ~= nil then								
									local overlay = self.panelOverlay.mainBackground;
									isOverlapped = checkOverlayOverlap(self.lastMouseXPos, self.lastMouseYPos, overlay);
								end;
								
								if not isOverlapped then
									v:onEnter(dt);
									self.foundInteractiveObject = v;
									break;
								end;
							end;
						end;
					end;
				end;
			end;
		end;
		
	end;
	
end;

function InteractiveControl:draw()	
	if self.isMouseActive then
		for k,v in pairs(self.interactiveObjects) do
			v:draw();
		end;
		g_currentMission:addHelpButtonText(g_i18n:getText("InteractiveControl_Off"), InputBinding.INTERACTIVE_CONTROL_SWITCH);
	else
		g_currentMission:addHelpButtonText(g_i18n:getText("InteractiveControl_On"), InputBinding.INTERACTIVE_CONTROL_SWITCH);
	end;
end;

function InteractiveControl:onLeave()
	self.cameras[self.indoorCamIndex].isActivated = true;
	setShowMouseCursor(false);
end;


function InteractiveControl:setPanelOverlay(panel)
	
	if self.panelOverlay ~= nil then
		if self.panelOverlay.setActive ~= nil then
			self.panelOverlay:setActive(false);
		end;
	end;
	self.panelOverlay = panel;

	if panel ~= nil then
		if panel.setActive ~= nil then
			panel:setActive(true);
		end;
	end;
end;